#include <sstream>
#include <fstream>
#include "Shader.h"

#include "../Exceptions/IOException.h"
#include "../Exceptions/Shaders/ShaderCompileException.h"
#include "../exceptions/shaders/CreateProgramException.h"

namespace Nebulix 
{

	const char* vertexShaderSourceCode = "#version 330 core\n"
		"layout (location = 0) in vec3 aPos;\n"
		"void main()\n"
		"{\n"
		"   gl_Position = vec4(aPos.x, aPos.y, aPos.z, 1.0);\n"
		"}\0";
	const char* fragmentShaderSourceCode = "#version 330 core\n"
		"out vec4 FragColor;\n"
		"void main()\n"
		"{\n"
		"   FragColor = vec4(1.0f, 0.5f, 0.2f, 1.0f);\n"
		"}\n\0";

	Shader::Shader(std::string& vertexShaderfile, std::string& fragmentShaderfile): _vertexFile{vertexShaderfile}, _fragmentFile{fragmentShaderfile}
	{
		std::ifstream vertexShaderSource(vertexShaderfile), fragmentShaderSource(fragmentShaderfile);
		GLuint vertexShader = 0, fragmentShader = 0;

		CompileShadercode(vertexShaderSource, vertexShader, ShaderType::Vertex);
		CompileShadercode(fragmentShaderSource, fragmentShader, ShaderType::Fragment);

		CreateProgram(vertexShader, fragmentShader);
	}

	void Shader::CompileShadercode(std::ifstream& shaderSource, GLuint& shaderObject, ShaderType type)
	{
		if (!shaderSource.is_open())
		{
			throw IOException("ERROR::SHADER::CANNOT_OPEN_FILE");
		}

		const char* shader;
		
		shaderSource.exceptions(std::ifstream::failbit | std::ifstream::badbit);
		try
		{
			std::stringstream sourceCode;
			sourceCode << shaderSource.rdbuf();
			shaderSource.close();
			std::string code = sourceCode.str();
			shader = code.c_str();
		}
		catch (const std::exception&)
		{
			throw Nebulix::IOException("ERROR::SHADER::READ_FILE");
		}
		

		if (type == ShaderType::Vertex)
			shaderObject = glCreateShader(GL_VERTEX_SHADER);
		else if (type == ShaderType::Fragment)
			shaderObject = glCreateShader(GL_FRAGMENT_SHADER);
		else
			throw std::runtime_error("ERROR::SHADER::TYPE \nCannot create shader program with the given shader type '" + std::to_string((int)type) + "'");

		glShaderSource(shaderObject, 1, &shader, NULL);
		glCompileShader(shaderObject);

		int success;
		char infoLog[512];
		glGetShaderiv(shaderObject, GL_COMPILE_STATUS, &success);
		if (!success)
		{
			glGetShaderInfoLog(shaderObject, 512, NULL, infoLog);
			throw Nebulix::ShaderCompileException(type, infoLog);
		}
	}

	void Shader::CreateProgram(GLuint& vertexShader, GLuint& fragmentShader)
	{
		shaderId = glCreateProgram();
		glAttachShader(shaderId, vertexShader);
		glAttachShader(shaderId, fragmentShader);
		glLinkProgram(shaderId);

		int success;
		char infoLog[512];
		glGetProgramiv(shaderId, GL_LINK_STATUS, &success);
		if (!success) {
			glGetProgramInfoLog(shaderId, 512, NULL, infoLog);
			throw Nebulix::CreateProgramException(infoLog);
		}

		glDeleteShader(vertexShader);
		glDeleteShader(fragmentShader);
	}
}
