#pragma once
#include <fstream>
#include <vector>
#include <string>
#include <glad/glad.h>
#include "Enums.h"


namespace Nebulix 
{
	class Shader
	{
	public:
		Shader() = delete;
		Shader(std::string& vertexShaderfile, std::string& fragmentShaderfile);
		~Shader() 
		{
			glDeleteProgram(shaderId);
		}

		/// <summary>
		/// To actually use and activate the shader, call this.
		/// </summary>
		void Use() { glUseProgram(shaderId); }

		void setBool(const std::string& name, bool value) const 
		{
			glUniform1i(glGetUniformLocation(shaderId, name.c_str()), (int)value);
		}
		void setInt(const std::string& name, int value) const
		{
			glUniform1i(glGetUniformLocation(shaderId, name.c_str()), value);
		}
		void setFloat(const std::string& name, float value) const
		{
			glUniform1f(glGetUniformLocation(shaderId, name.c_str()), value);
		}

	private:
		std::string &_vertexFile, &_fragmentFile;
		GLuint shaderId;

		void CompileShadercode(std::ifstream& shaderSource, GLuint& shaderId, ShaderType type);
		void CreateProgram(GLuint& vertexShader, GLuint& fragmentShader);
	};
}
